clear system
clear options
clear sequence

% the spectral sampling
c=1.4;      % center frequency of Gaussian distribution of spins, in GHz
g=0.01;     % width of the Gaussian distribution in GHz
nu0_s=1.3;  % starting value for sampling of Gaussian distribution
nu0_e=1.5;  % final value for sampling of Gaussian distribution
st=0.002;   % stepsize for sampling of Gaussian distribution
nu0_vec=nu0_s:st:nu0_e;     % creates vector with resonance frequencies, according to above
p=exp(-((c-nu0_vec)/g).^2); % probabilities for each spin
p=p/trapz(p);

% the system
system.sqn=[0.5 0.5];       % spin quantum numbers
system.interactions={1,0, 'z','e',1.4;     % Zeeman term of electron, value is replaced in loop
                     2,0,'z','e',-0.014;   % Zeeman of nucleus
                     1,2, 'z','z',-0.045;  % secular part of hyperfine interaction , A*SzIz
                     1,2, 'z','x',0.008};  % pseudo secular part of hyperfine interaction , B*SzIz
system.init_state='-ze'; % only the electron spin is quantizised along z, nucleus is not defined, same statement would be '-z'
system.eq = 'init';  % equilibrium state is the same as initial state

% the options
options.relaxation=0;       % tells SPIDYAN whether to include relaxation (1) or not (0)
options.down_conversion=0;  % downconversion of signal (1) or not (0)
options.det_op={'pe'};      % detection operator

% the sequence
sequence.tp=[50,300,25,600];  % vector with event lengths in ns
sequence.beta(1)=pi/2;        % flip angle for first event, tells SPIDYAN that 1st event is a pulse
sequence.beta(3)=pi;          % flip angle for second event, tells SPIDYAN that 2nd event is a pulse
sequence.frq={[1.3 1.5]};     % initial and final frequency of the chirp
sequence.detection=ones(1,length(sequence.tp)); % detection switched on for fourth event
sequence.t_rise=10;           % rise time of chirp pulses
sequence.excite={[1 0]};      % only excite the first spin during the pulses

[experiment,options] = triple(sequence, options); % builds experiment (pulses)

signalc=cell(1,length(nu0_vec)); % creates an empty cell to store results


parfor k=1:length(nu0_vec)
    
    systeml=system;                          % copy system into loop
    systeml.interactions{1,end}= nu0_vec(k); % add loop resonance frequency 
       
    [systeml, state, optionsl]=setup(systeml,options); % build system 
    
    [state, detected_signal, experimentl]=homerun(state,systeml,experiment, optionsl,[]);  % propagate
    
    signalc{k}=detected_signal.sf;   % keep signals from simulation frame
end


signal=zeros(size(signalc{1}));                  % creates empty signal
t=linspace(0,sum(experiment.tp),length(signal)); % creates time axis

for k=1:length(nu0_vec)
    signal=signal+signalc{k}*p(k);      % combines all signals
end

figure(1); clf; plot(t,real(signal)) 
title('Signal in simulation frame')
xlabel('time [ns]')
ylabel('<S_{1p}>')

signaldc=strike(signal, t, c, options);   % down conversion of merged signals
signaldc=signaldc/max(max(signaldc));

figure(2); clf; 
title('Signal after down conversion')
hold on
plot(t,real(signaldc))
xlabel('time [ns]')
ylabel('<S_{1p}>')
plot([experiment.tp(1) experiment.tp(1)],[-1 1],'r--')
plot([experiment.tp(2)+experiment.tp(1) experiment.tp(2)+experiment.tp(1)],[-1 1],'r--')
plot([experiment.tp(3)+experiment.tp(2)+experiment.tp(1) experiment.tp(3)+experiment.tp(1)+experiment.tp(2)],[-1 1],'r--')
