function [gamma] = changeup(system)
%  [gamma] = changeup(system)
%
%  The function changeup calculates the relaxation superoperators for a 
%  given spinsystem system.sqn, which is a vector of spin quantum numbers.
%  Relaxation times can either be given as a single or in matrix form. 
%  In the former case, the same value is applied all T1 or T2 relaxation
%  pathways:
%  
%  system.T1 = 1000;
%  system.T2 = 500;
%
%  The latter allows to define T1 and T2 values to all transitions 
%  seperately. The used matrix has the same structure as a denisty matrix
%  of the system, with elements corresponding to the same states as in
%  sigma. The off-diagonal elements correlate the pathways between the
%  states in the first dimension and the second dimension. Therefore it is
%  necessary to define the upper triangle only.
% 
%  Assign different relaxation times to all transitions in above example:
% 
%                aa  ab     ba      bb
%  system.T1 =  [0   10^7   10^3    10^4;  aa
%                0   0      10^4    10^3;  ab
%                0   0      0       10^7;  ba
%                0   0      0       0   ;] bb
% 
%                aa  ab     ba      bb
%  system.T2 =  [0   10^3   10^2    10^3;  aa
%                0   0      10^3    10^2;  ab
%                0   0      0       10^5;  ba
%                0   0      0       0   ;] bb
% 
%  From this, the program builds the relaxation superator gamma in 
%  Liouville space.
%
%  St. Pribitzer, 2015

[t11, t12]=size(system.T1);
[t21, t22]=size(system.T2);

dim = prod(system.sqn*2+1);

T1ud = 0;

% checks the format of the input and expands it to matrices if necessary
% for T1 times, which allows for easier processing
if t11==1 && t12==1
    T1 = system.T1*ones(dim); 
else
    T1=system.T1;
    if any(any(tril(T1)))
       T1ud =1; 
    end
end

% checks the format of the input and expands it to matrices if necessary
% for T2 times, which allows for easier processing
if t21~=1 || t22~=1
    T2=system.T2;
    if ~any(any(tril(T2)))
        % mirrors upper triangle of matrix to lower triangle
        T2=triu(T2)+triu(T2,1)';
    end
elseif t21==1 && t22==1 
    T2 = system.T2*ones(dim);   
end

% Assigns default values or inf to all T1 relaxation paths which are zero.
if isfield(system,'default_T1') && ~isempty(system.default_T1)
    T1(T1==0)=system.default_T1;
else
    T1(T1==0) = 1000000;
end

% Assigns default values or inf to all T1 relaxation paths which are zero.
if isfield(system,'default_T2') && ~isempty(system.default_T2)
    T2(T2==0)=system.default_T2;
else 
    T2(T2==0) = 1000000;
end

nh = dim;
gamma=zeros((nh)^2);

kk=1;
jj=2;

% calculates the positions of longitudinal relaxation, populations are
% never on the diagonal elements of the relaxation superoperator.
for k1=1:nh+1:nh^2
    
    for k2=k1+nh+1:nh+1:nh^2
        
        gamma(k1,k2)=-1/T1(kk,jj);
        
        if T1ud
            gamma(k2,k1)=-1/T1(jj,kk);
        else
            gamma(k2,k1)=-1/T1(kk,jj);
        end
        % since we are using only reduced density matrices (difference of
        % the equilibrium state to the current state), it is not necessary
        % to compensate the T1 decay of off-diagonal elements/states with
        % the diagonal elements:
%         gamma(k1,k1)=gamma(k1,k1)+1/T1(kk,jj);
%         gamma(k2,k2)=gamma(k2,k2)+1/T1(kk,jj);
        jj=jj+1;
        
    end
    
    kk=kk+1;
    jj=kk+1;
    
end

% populates positions of transverse relaxation elements
n=1;
% rewrites T2 matrix into a vector, to allow effective processing
T2vec=reshape(T2,nh*nh,1);
for k=2:nh^2-1
    
    if k~=n+1+n*nh
        gamma(k,k)=1/T2vec(k);
    else
        n=n+1;
    end 
end


end

